import React from "react";

const clamp = ({ left, top, width, height }) => {
  if (left < 0) {
    left = 0;
  } else if (left + width > document.documentElement.clientWidth) {
    left = document.documentElement.clientWidth - width;
  }

  // TODO clamp top based on a prop

  return { left, top };
};

export default class extends React.Component {
  constructor(props) {
    super(props);
    this.refContainer = React.createRef();
    this.refDropdown = React.createRef();

    this.state = {
      position: { left: 0, top: 0 },
      isDropdownShown: false,
    };
  }

  componentWillUnmount() {
    window.removeEventListener("scroll", this.calculatePosition);
  }

  calculatePosition() {
    // console.log("calcing");
    const { left, top, /*width,*/ height } =
      this.refContainer && this.refContainer.current.getBoundingClientRect();

    const clamped = clamp({
      left,
      top,
      width:
        this.refDropdown &&
        this.refDropdown.current.getBoundingClientRect().width,
      height:
        this.refDropdown &&
        this.refDropdown.current.getBoundingClientRect().height,
    });
    this.setState({
      position: { left: clamped.left, top: clamped.top + height },
    });
  }

  calculatePositionThenShow() {
    this.calculatePosition();
    this.setState({ isDropdownShown: true });
    window.addEventListener("scroll", this.calculatePosition);
  }

  close() {
    this.setState({ isDropdownShown: false });
    window.removeEventListener("scroll", this.calculatePosition);
  }

  render() {
    return (
      <div
        ref={this.refContainer}
        onMouseEnter={this.calculatePositionThenShow}
        onMouseLeave={this.close}
        style={{ height: "fit-content", width: "fit-content" }}
      >
        {this.props.children}
        <div
          ref={this.refDropdown}
          style={{
            visibility: this.state.isDropdownShown ? "visible" : "hidden",
            position: "fixed",
            top: this.state.position.top,
            left: this.state.position.left,
          }}
        >
          {this.props.dropdown}
        </div>
      </div>
    );
  }
}
